assert(loadfile("code/tabloader.lua"))()

print("Loading tools DB...")
local toolDB = loadTabFile("data/blueprints_tools.csv", { "damage", "quantity", "item", "tool" }, ",", 3 )
print("Loading materials DB...")
local matDB = loadTabFile("data/blueprints_complex.csv", { "quantity", "material", "item" }, ",", 3 )

local bptFile = io.open("data/cache/blueprints_tools.dat", "w")
local t1File = io.open("data/cache/blueprints_t1.dat", "w")
local complexFile = io.open("data/cache/blueprints_complex.dat", "w")

local t1types = { "Tritanium", "Pyerite", "Mexallon", "Isogen", "Nocxium", "Zydrine", "Megacyte" }

local t1mats = { }

for _,itemName in ipairs(t1types) do
	t1mats[itemName] = true
end

local items = { }

-- Item format:
--    materials: Materials keyed by type
--    tools: Tool damage keyed by type
--    t1: Can be stored as a T1 item

local function getItem(name)
	local i = items[name]

	if i == nil then
		i = { materials = { }, tools = { }, t1 = true }
		items[name] = i
	end

	return i
end

for _,row in ipairs(matDB.rows) do
	local item = getItem(row.item)

	if not t1mats[row.material] then
		item.t1 = false
	end

	item.materials[row.material] = row.quantity
end

for _,row in ipairs(toolDB.rows) do
	local item = getItem(row.item)

	item.t1 = false
	item.tools[row.tool] = row.damage * row.quantity

	if row.damage == 1 then
		-- Consumes base items
		local baseItem = items[row.tool]

		if baseItem then
			for baseMat,qty in pairs(baseItem.materials) do
				if item.materials[baseMat] then
					item.materials[baseMat] = (item.materials[baseMat] or 0) - (qty*row.quantity)
				end
			end
		end
	end
end

-- Dump tools
bptFile:write("item\ttool\tdamage\n")

for itemName,item in pairs(items) do
	for toolName,toolDamage in pairs(item.tools) do
		bptFile:write(itemName.."\t"..toolName.."\t"..toolDamage.."\n")
	end
end
bptFile:close()

-- Dump T1
t1File:write("item\ttritanium\tpyerite\tmexallon\tisogen\tnocxium\tzydrine\tmegacyte\n")
for itemName,item in pairs(items) do
	if item.t1 then
		t1File:write(itemName)
		for _,itemName in ipairs(t1types) do
			t1File:write("\t"..(item.materials[itemName] or "0"))
		end
		t1File:write("\n")
	end
end
t1File:close()

-- Dump complex
complexFile:write("item\tmaterial\tquantity\n")
for itemName,item in pairs(items) do
	if not item.t1 then
		for matName,quantity in pairs(item.materials) do
			if quantity > 0 then
				complexFile:write(itemName.."\t"..matName.."\t"..quantity.."\n")
			end
		end
	end
end
complexFile:close()

local toolDB = nil
local matDB = nil

-- Dump multiple quantity
local mqDB = loadTabFile("data/blueprints_multiplequantity.csv", { "item", "quantity" }, ",", 3 )

local mqFile = io.open("data/cache/blueprints_multiple_quantity.dat", "w")
mqFile:write("item\tquantity\n")
for _,row in pairs(mqDB.rows) do
	mqFile:write(row.item.."\t"..row.quantity.."\n")
end
mqFile:close()
mqDB = nil

-- Dump properties
print("Loading properties DB...")
local propDB = loadTabFile("data/blueprints_properties.csv", { "item", "copyTime", "inventTime", "buildTime", "maxRuns" }, ",", 3 )

local propFile = io.open("data/cache/blueprints_properties.dat", "w")
propFile:write("item\tcopyTime\tinventTime\tbuildTime\tmaxRuns\n")
for _,item in ipairs(propDB.rows) do
	-- For some reason, the DB values are the time needed to produce half of the production limit
	-- :ccp:
	local actualCopyTime = tonumber(item.copyTime) * 2 / tonumber(item.maxRuns)
	propFile:write(item.item.."\t"..actualCopyTime.."\t"..item.inventTime.."\t"..item.buildTime.."\t"..item.maxRuns.."\n")
end
propFile:close()

-- Hard-wired chance tables
local veryHighChance = { }
setmetatable(veryHighChance, { __index = function(k,v) return 0.4 end })

local highChance = { }
setmetatable(highChance, { __index = function(k,v) return 0.3 end })
local medChance = { }
setmetatable(medChance, { __index = function(k,v) return 0.25 end })
local lowChance = { }
setmetatable(lowChance, { __index = function(k,v) return 0.2 end })

local chances = {
	Ship =
	{
		-- BC/BS
		Marauder = lowChance,
		["Black Ops"] = lowChance,
		["Command Ship"] = lowChance,

		["Combat Recon Ship"] = medChance,
		["Force Recon Ship"] = medChance,
		Logistics = medChance,
		["Transport Ship"] = medChance,
		["Heavy Assault Ship"] = medChance,
		["Heavy Interdictor"] = medChance,

		Interceptor = highChance,
		Interdictor = highChance,
		["Electronic Attack Ship"] = highChance,
		["Assault Ship"] = highChance,
		["Stealth Bomber"] = highChance,
		["Jump Freighter"] = highChance,
		["Covert Ops"] = highChance,

		Exhumer =
		{
			Hulk = 0.2,
			Mackinaw = 0.25,
			Skiff = 0.3
		}
	}
}

local veryHighCategory = { }
setmetatable(veryHighCategory, { __index = function(k,v) return veryHighChance end })

setmetatable(chances, { __index = function(k,v) return veryHighCategory end })

local interfaceRaces =
{
	Occult = "Amarr",
	Incognito = "Gallente",
	Esoteric = "Caldari",
	Cryptic = "Minmatar",
}

local t2DB = loadTabFile("data/blueprints_t2.csv", { "parentTypeName", "childTypeName", "groupName", "categoryName" }, ",", 3 )
local dcDB = loadTabFile("data/blueprints_datacores.csv", { "bpTypeName", "reqTypeName", "quantity" }, ",", 3 )

local t1inventReqs = { }

for _,row in ipairs(dcDB.rows) do
	local bp = t1inventReqs[row.bpTypeName]

	if bp == nil then
		bp = { inventedFrom = row.bpTypeName }
		t1inventReqs[row.bpTypeName] = bp
	end

	local interfaceType = string.match(row.reqTypeName, "(.-) .+")

	if interfaceType == "Datacore" then
		if bp.datacore1 == nil then
			bp.datacore1 = row.reqTypeName
			bp.datacore1quantity = row.quantity
		else
			bp.datacore2 = row.reqTypeName
			bp.datacore2quantity = row.quantity
		end
	else
		bp.race = interfaceRaces[interfaceType]
	end
end

local inventFile = io.open("data/cache/blueprints_invention.dat", "w")
inventFile:write("item\tinventedFrom\tbaseChance\trace\tdatacore1\tdatacore1quantity\tdatacore2\tdatacore2quantity\n")
for _,row in ipairs(t2DB.rows) do
	local t2item = { item = row.childTypeName }

	-- Clone the T1 table
	local parentTable = t1inventReqs[row.parentTypeName]

	-- Might be nil if the base item isn't actually inventable (i.e. Micro Capacitor Batteries)
	if parentTable ~= nil then
		for k,v in pairs(parentTable) do
			t2item[k] = v
		end

		-- Add invent chance
		t2item.baseChance = chances[row.categoryName][row.groupName][row.childTypeName]

		-- Might be nil race for civilian datacore items (i.e. Perpetual Motion Machines)
		if t2item.race ~= nil then
			inventFile:write(
				t2item.item.."\t"..
				t2item.inventedFrom.."\t"..
				t2item.baseChance.."\t"..
				t2item.race.."\t"..
				t2item.datacore1.."\t"..
				t2item.datacore1quantity.."\t"..
				t2item.datacore2.."\t"..
				t2item.datacore2quantity.."\n")
		end
	end
end
inventFile:close()

inventionJobs = nil